// FTFilter.h
/*
	Copyright  1991-1995 by TopSoft, Inc.  All Rights Reserved Worldwide.
	Distribution restricted by the accompanying TopSoft License version 1.1.
	FilterTop is a trademark of TopSoft, Inc.

PRODUCT
        FilterTop 1.0

FILE
        FTFilter.h

DESCRIPTION
        This file contains the header information needed for filter development.
        This file must be included by all filters.  This file should should be
        independent of all other FilterTop header files and should not rely on
        FilterTop conventions (such as precompiled MacHeaders).  This file should
        not contain definitions that are unused by filter developers.
        
DEVELOPED BY
        Jim Brunner                            jim@brunner.wf.com

CARETAKER - Jim Brunner <jim@brunner.wf.com>
 Please consult this person for any changes or suggestions to this file.

MODIFICATION HISTORY
        This file was developed by the FilterTop development team, specifcally 
        this file was written by Chris Priestley, Peter Lewis and Doug Yarrington
        in early 1992.  In June of 93, this file was rewritten from scratch to
        implement the new FilterTop API as described in "Functional Description
        for Filter API 2.0"  By Jim Brunner.
	 
        dd mmm yy  -  xxx  -  x.xRx:  description of patch
        28 May 93  -  JHB  -  patch history to API version 1 removed
        15 Jun 93  -  JHB  -  version 2 of the API implemented
        11 Oct 93  -  TJ   -  added fINF error.
        17 Apr 94  -  JHB  -  Header comment updated
        15 Oct 94  -  JHB  -  Added A4 defines for MWERKS
		29 Mar 95  -  JHB  -  Added A4 defines for Think C
        29 Aug 95  -  JHB  -  Update license info in header
*/


#ifndef	__FTFILTER__
#define	__FTFILTER__

#include	<Types.h>
#include	<Files.h>
#include	<Dialogs.h>
#include	<Events.h>

//--------------------------------------------------------------------
// Message codes for messages sent to a filter...
//--------------------------------------------------------------------

typedef long FilterMessage;

#define	msgOpenFilter			1000L	// tell filter to initialize itself
#define	msgCloseFilter			1010L	// tell filter to finish
#define	msgFilterData			2100L	// process the input stream
#define	msgInitOptions			2300L	// self-config start
#define	msgDialogEvent			2310L	// self-config handle DLOG event
#define	msgCloseOptions			2320L	// self-config clean up

//--------------------------------------------------------------------
// Error codes returned by callbacks
//--------------------------------------------------------------------

typedef long FTErr;

#define	ftOK				0L	// OK.  No error.
#define	ftAbort				1L	// User wants abort - clean up processing and get out
#define	ftParmError			2L	// parameter error in function call
#define	ftParsetabInvalid	3L	// the indicated parse table was invalid
#define	ftPipeInvalid		4L	// the indicated pipe identifier is not valid
#define	ftStreamInvalid		5L	// the indicated stream identifier is not valid
#define	ftStreamNotInput	6L	// the indicated stream must be input (and isn't)
#define	ftStreamNotOutput	7L	// the indicated stream must be output (and isn't)
#define	ftEndOfData			8L	// there is no more data for this input stream
#define	ftResNotFound		9L	// the indicated resource does not exist in given stream
#define	ftFileExists		10L	// the output file already exists
#define ftFileDoesntExist	11L	// the input file doesnt exist
#define	ftIOError			12L	// the Mac file system has reported an I/O error

//--------------------------------------------------------------------
// Pipe identifiers.  Currently there are up to one input and 4 outputs.
//--------------------------------------------------------------------

typedef long Pipe;

#define	input1	 0x0100L
#define	errpipe	 0x0200L
#define	output1	 0x0300L
#define	output2  0x0301L	// 1.0a2 SJ addition
#define	output3  0x0302L	// 1.0a2 SJ addition

//--------------------------------------------------------------------
// System defined parse tables for ParseRead
//--------------------------------------------------------------------

//#define	brkEOL	 256L		// These tables not defined yet...
//#define	brkWhite 257L
//#define	brkAlpha 258L
//#define	brkNum   259L

//--------------------------------------------------------------------
// Type definitions needed for the callbacks
//--------------------------------------------------------------------

typedef long Stream;

//--------------------------------------------------------------------
// Struct definition for the callbacks
//--------------------------------------------------------------------

typedef struct
{
	// routines for opening and closing streams...
	pascal FTErr (*Open)			(Pipe pipe, Stream *stream,
									 FInfo *fndrinfo, FSSpec *suggested_spec);
	pascal FTErr (*Close)			(Stream stream);

	// routines for reading and writing the data portion of streams...	
	pascal FTErr (*Read)			(Stream stream, void *buffer, long *count);
	pascal FTErr (*ParseRead)		(Stream stream, long parseTab_or_char,
									 void *buffer, long *count, short *breakchar);
	pascal FTErr (*GetParseTable)	(long *parsetable, unsigned char *breakchars,
									 unsigned char *omitchars, unsigned char *parsemodes);
	pascal FTErr (*FreeParseTable)	(long parsetable);
	pascal FTErr (*Write)			(Stream stream, void *buffer, long count);
	pascal FTErr (*WriteString)		(Stream stream, Str255 string);
	pascal FTErr (*CopyData)		(Stream from_stream, Stream to_stream);

	// routines for reading and writing the resource portion of streams...
	pascal FTErr (*CountTypes)		(Stream stream, short *count);
	pascal FTErr (*GetIndType)		(Stream stream, short index, ResType *theType);
	pascal FTErr (*CountResources)	(Stream stream, ResType theType, short *count);

	pascal FTErr (*GetIndResource)	(Stream stream, ResType theType, short index,
									 short *theID, Str255 name, short *attrs, Handle *theResource);
	pascal FTErr (*GetResource)		(Stream stream, ResType theType, short theID,
									 Str255 name, short *attrs, Handle *theResource);
	pascal FTErr (*GetNamedResource)(Stream stream, ResType theType, Str255 name,
									 short *theID, short *attrs, Handle *theResource);

	pascal FTErr (*ReleaseResource)	(Handle theResource);
	pascal FTErr (*UniqueID)		(Stream stream, ResType theType, short *theID);
	pascal FTErr (*PutResource)		(Stream stream, ResType theType, short theID,
									 Str255 name, short attrs, Handle theResource);
	pascal FTErr (*CopyResources)	(Stream from_stream, Stream to_stream);

//	routines to be implemented after initial implementation (release 1 only if time permits)
//	// routines for converting a stream to a file and vice versa...
	pascal FTErr (*StreamToFile)	(Stream from_stream, FSSpec to_file);
	pascal FTErr (*FileToStream)	(FSSpec from_file, Stream to_stream);

	// error reporting
	pascal FTErr (*ReportError)		(FTErr errorCode, Str255 errorString);

	// routines for auto-config
	pascal FTErr (*CfgFindData)		(OSType type, void *buffer, long *count);

	// routines for yielding time to other processes
	pascal FTErr (*Yield)			(void);
} 
	Callbacks;


//--------------------------------------------------------------------
// Definition of parameter block passed to filters
//--------------------------------------------------------------------

typedef struct
{
        FilterMessage	serviceRequested;	// message for filter to process
        long 			filterShellVersion;	// same format as in 'vers' resource 
        Callbacks		*cb;  				// callback parameter block
        DialogPtr		dlg; 				// dialog
        EventRecord		*event; 			// dialog event record 
        Handle			filterGlobalsH;   	// globals for this filter
        Handle			filterConfigH;		// config resource handle
        long			data; 				// generic data
} PBCallToFilter, *PBCallToFilterPtr;


//--------------------------------------------------------------------
// Definition of errors that can be returned by filters
//--------------------------------------------------------------------

typedef long FilterResult;

#define	filterOK				0L		// filter completed successfully
#define	filterMsgNotHandled		1L		// filter received an unknown message (this is OK)
#define	filterProcessingError	2L		// filter encountered a serious processing error
#define	filterAborted			3L		// filter processing was aborted


//--------------------------------------------------------------------
// Definition of a filter proc
//--------------------------------------------------------------------

typedef pascal FilterResult FilterProc(PBCallToFilterPtr pb);


//--------------------------------------------------------------------
// Definition of a ENTER_FILTER and EXIT_FILTER() defines
//--------------------------------------------------------------------

#ifdef __MWERKS__
#if (__A5__ == 0)
#include <A4Stuff.h>

// Call this #define at the entry point of the filter.  This allows the filter
//  access to strings, etc. that the compiler has generated as globals.  (Do not use
//  global variables in your filter though because filters need to be reentrant.)
#define ENTER_FILTER	long FToldA4 = SetCurrentA4()

// Call this #define at exit points of the filter.  This MUST be called
//  from the same routine that called ENTER_FILTER.  Pass the FilterResult
//  to be returned as the single parameter.
#define EXIT_FILTER(rc)	{										\
							FilterResult	tempRC = (rc);		\
							SetA4(FToldA4);						\
							return tempRC;						\
						}
#endif
#elif defined(THINK_C)
#if __option(a4_globals)
#include <setupa4.h>
#define ENTER_FILTER	RememberA0();  SetUpA4()
#define EXIT_FILTER(rc)	{										\
							FilterResult	tempRC = (rc);		\
							RestoreA4();						\
							return tempRC;						\
						}
#endif
#else
// If other C/C++ compilers are used to compile filters, the filter writer
//  had better not use ENTER_FILTER or EXIT_FILTER because we haven't 
//  defined them.  A syntax error will be generated.
#endif

#endif
